import IconButton from '@material-ui/core/IconButton'
import GraphIcon from 'assets/icons/ico/graph-icon.svg'
import LeftArrowIcon from 'assets/icons/ico/left-arrow.svg'
import RightArrowIcon from 'assets/icons/ico/right-arrow.svg'
import StyledIcon from 'components/CommonKit/Icon/StyledIcon'
import Loader from 'components/Loader/Loader'
import { useClient } from 'cozy-client'
import { useI18n } from 'cozy-ui/transpiled/react/I18n'
import Icon from 'cozy-ui/transpiled/react/Icon'
import { FluidType } from 'enum/fluid.enum'
import { TimeStep } from 'enum/timeStep.enum'
import { DateTime } from 'luxon'
import { Dataload, TimePeriod } from 'models'
import React, { useCallback, useEffect, useState } from 'react'
import { useSelector } from 'react-redux'
import ConsumptionDataManager from 'services/consumption.service'
import { AppStore } from 'store'
import { getNavPicto } from 'utils/picto'
import { formatNumberValues } from 'utils/utils'
import './maxConsumptionCard.scss'

interface MaxConsumptionCardProps {
  analysisDate: DateTime
}

const MaxConsumptionCard: React.FC<MaxConsumptionCardProps> = ({
  analysisDate,
}: MaxConsumptionCardProps) => {
  const { t } = useI18n()
  const client = useClient()

  const { fluidTypes } = useSelector((state: AppStore) => state.ecolyo.global)
  const [index, setIndex] = useState<number>(0)

  const [maxDayData, setMaxDayData] = useState<Dataload | null>(null)
  const [isLoading, setIsLoading] = useState<boolean>(false)

  const handleChangePrevFluid = useCallback(() => {
    setIsLoading(true)
    if (index === 0) {
      setIndex(fluidTypes.length - 1)
    } else {
      setIndex(prev => prev - 1)
    }
  }, [fluidTypes, index])

  const handleChangeNextFluid = useCallback(() => {
    setIsLoading(true)
    if (index === fluidTypes.length - 1) {
      setIndex(0)
    } else {
      setIndex(prev => prev + 1)
    }
  }, [fluidTypes, index])

  useEffect(() => {
    let subscribed = true
    async function getMaxLoadData() {
      setIsLoading(true)
      const timePeriod: TimePeriod = {
        startDate: analysisDate.minus({ month: 1 }).startOf('month'),
        endDate: analysisDate.minus({ month: 1 }).endOf('month'),
      }
      const consumptionService = new ConsumptionDataManager(client)
      const monthMaxData = await consumptionService.getMaxLoad(
        timePeriod,
        TimeStep.DAY,
        [fluidTypes[index]],
        undefined,
        false,
        true
      )
      if (monthMaxData) {
        setMaxDayData(monthMaxData as Dataload)
      } else {
        setMaxDayData(null)
      }
      setIsLoading(false)
    }
    if (subscribed) {
      getMaxLoadData()
    }
    return () => {
      subscribed = false
    }
  }, [analysisDate, client, fluidTypes, index])
  return (
    <div className="max-consumption-container">
      <StyledIcon icon={GraphIcon} size={38} />
      <div className="text-16-normal title">{t('analysis.max_day')}</div>
      <div className="fluid-navigation">
        {fluidTypes.length > 1 && (
          <IconButton
            aria-label={t('consumption.accessibility.button_previous_value')}
            onClick={handleChangePrevFluid}
            className="arrow-prev"
          >
            <Icon icon={LeftArrowIcon} size={24} />
          </IconButton>
        )}
        <div
          className={`text-20-bold fluid ${FluidType[
            fluidTypes[index]
          ].toLowerCase()}`}
        >
          {`${t('FLUID.' + FluidType[fluidTypes[index]] + '.LABEL')}`}
        </div>
        {fluidTypes.length > 1 && (
          <IconButton
            aria-label={t('consumption.accessibility.button_previous_value')}
            onClick={handleChangeNextFluid}
            className="arrow-next"
          >
            <Icon icon={RightArrowIcon} size={24} />
          </IconButton>
        )}
      </div>
      <div className="data-container">
        {isLoading && <Loader fluidType={fluidTypes[index]} />}
        {!isLoading && (
          <>
            {!maxDayData && (
              <p className={`text-20-bold no_data`}>{t('analysis.no_data')}</p>
            )}
            {maxDayData && (
              <>
                <div className="text-24-bold maxDay-date">
                  {maxDayData.date.setLocale('fr').toFormat('cccc dd LLLL')}
                </div>
                <Icon
                  className="dataloadvisualizer-euro-fluid-icon"
                  icon={getNavPicto(fluidTypes[index], true, true)}
                  size={38}
                />
                <div className="maxDay-load">
                  {formatNumberValues(
                    maxDayData.value,
                    FluidType[fluidTypes[index]]
                  )}{' '}
                  {t(
                    `FLUID.${FluidType[fluidTypes[index]]}.${
                      maxDayData.value >= 1000 ? 'MEGAUNIT' : 'UNIT'
                    }`
                  )}
                </div>
              </>
            )}
          </>
        )}
      </div>
    </div>
  )
}

export default MaxConsumptionCard
