import IconButton from '@material-ui/core/IconButton'
import LeftArrowIcon from 'assets/icons/ico/left-arrow.svg'
import RightArrowIcon from 'assets/icons/ico/right-arrow.svg'
import classNames from 'classnames'
import DateNavigatorFormat from 'components/DateNavigator/DateNavigatorFormat'
import { useI18n } from 'cozy-ui/transpiled/react/I18n'
import Icon from 'cozy-ui/transpiled/react/Icon'
import { FluidType } from 'enum/fluid.enum'
import { TimeStep } from 'enum/timeStep.enum'
import { DateTime } from 'luxon'
import React, { Dispatch } from 'react'
import { useDispatch, useSelector } from 'react-redux'
import DateChartService from 'services/dateChart.service'
import { AppActionsTypes, AppStore } from 'store'
import { setCurrentIndex, setSelectedDate } from 'store/chart/chart.slice'
import { isLastDateReached } from 'utils/date'
import { isKonnectorActive } from 'utils/utils'
import './datenavigator.scss'

interface DateNavigatorProps {
  currentAnalysisDate?: DateTime
  setCurrentAnalysisDate?: React.Dispatch<React.SetStateAction<DateTime>>
  inlineDateDisplay?: boolean
}

const DateNavigator: React.FC<DateNavigatorProps> = ({
  currentAnalysisDate,
  setCurrentAnalysisDate,
  inlineDateDisplay = false,
}: DateNavigatorProps) => {
  const { t } = useI18n()
  const dispatch = useDispatch<Dispatch<AppActionsTypes>>()
  const {
    chart: { currentTimeStep, selectedDate, currentIndex },
    global: { fluidStatus },
  } = useSelector((state: AppStore) => state.ecolyo)

  const disablePrev =
    selectedDate <
    DateTime.local(0, 1, 1).setZone('utc', {
      keepLocalTime: true,
    })
  const disableNext: boolean = currentAnalysisDate
    ? isLastDateReached(currentAnalysisDate, TimeStep.MONTH)
    : isLastDateReached(selectedDate, currentTimeStep)

  const dateChartService = new DateChartService()

  const handleClickMove = (increment: number) => {
    if (!currentAnalysisDate) {
      const updatedDate: DateTime = dateChartService.incrementeDate(
        currentTimeStep,
        selectedDate,
        increment
      )
      const updatedIndex: number = dateChartService.defineDateIndex(
        currentTimeStep,
        updatedDate
      )
      dispatch(setSelectedDate(updatedDate))
      dispatch(setCurrentIndex(updatedIndex))
    } else {
      const updatedDate: DateTime = dateChartService.incrementeDate(
        TimeStep.MONTH,
        currentAnalysisDate,
        increment
      )
      setCurrentAnalysisDate?.(updatedDate)
    }
  }

  const handleChangePrevIndex = () => {
    if (!disablePrev && isKonnectorActive(fluidStatus, FluidType.MULTIFLUID)) {
      const increment: number =
        dateChartService.defineIncrementForPreviousIndex(
          currentTimeStep,
          selectedDate,
          currentIndex
        )
      if (currentAnalysisDate) {
        handleClickMove(-1)
      } else handleClickMove(increment)
    }
  }
  const handleChangeNextIndex = () => {
    if (!disableNext && isKonnectorActive(fluidStatus, FluidType.MULTIFLUID)) {
      const increment: number = dateChartService.defineIncrementForNextIndex(
        currentTimeStep,
        selectedDate,
        currentIndex
      )
      if (currentAnalysisDate) {
        handleClickMove(1)
      } else handleClickMove(increment)
    }
  }

  return (
    <div className="date-navigator">
      <div>
        <IconButton
          aria-label={t('consumption.accessibility.button_previous_value')}
          className={classNames('date-navigator-button', {
            ['disable']:
              disablePrev ||
              !isKonnectorActive(fluidStatus, FluidType.MULTIFLUID),
          })}
          onClick={() => handleChangePrevIndex()}
        >
          <Icon icon={LeftArrowIcon} size={16} />
        </IconButton>
      </div>
      <DateNavigatorFormat
        timeStep={currentAnalysisDate ? TimeStep.MONTH : currentTimeStep}
        date={
          currentAnalysisDate
            ? currentAnalysisDate.minus({ month: 1 })
            : selectedDate
        }
        inline={inlineDateDisplay}
      />

      <div>
        <IconButton
          aria-label={t('consumption.accessibility.button_next_value')}
          className={classNames('date-navigator-button', {
            ['disable']:
              disableNext ||
              !isKonnectorActive(fluidStatus, FluidType.MULTIFLUID),
          })}
          onClick={() => handleChangeNextIndex()}
        >
          <Icon icon={RightArrowIcon} size={16} />
        </IconButton>
      </div>
    </div>
  )
}

export default DateNavigator
