import Dialog from '@material-ui/core/Dialog'
import EnedisIcon from 'assets/icons/ico/consent-outdated-enedis.svg'
import errorIcon from 'assets/icons/visu/data-nok.svg'
import successIcon from 'assets/icons/visu/data-ok.svg'
import classNames from 'classnames'
import Loader from 'components/Loader/Loader'
import connectionWaitingText from 'constants/connectionWaitingText.json'
import firstConnectionWaitingText from 'constants/firstConnectionWaitingText.json'
import {
  ERROR_EVENT,
  SUCCESS_EVENT,
} from 'cozy-harvest-lib/dist/models/flowEvents'
import { useI18n } from 'cozy-ui/transpiled/react/I18n'
import Icon from 'cozy-ui/transpiled/react/Icon'
import { FluidType } from 'enum/fluid.enum'
import { KonnectorError } from 'enum/konnectorError.enum'
import { shuffle } from 'lodash'
import { Account } from 'models'
import React, { useCallback, useEffect, useMemo, useState } from 'react'
import './konnectorModal.scss'
import KonnectorModalFooter from './KonnectorModalFooter'

interface KonnectorModalProps {
  open: boolean
  isUpdating: boolean
  isLogging: boolean
  state: string | null
  error: KonnectorError | null
  fluidType: FluidType
  handleCloseClick: (isSuccess?: boolean) => void
  handleAccountDeletion: () => Promise<void>
  account: Account | null
}

const KonnectorModal: React.FC<KonnectorModalProps> = ({
  open,
  isUpdating,
  isLogging,
  state,
  error,
  fluidType,
  handleCloseClick,
  handleAccountDeletion,
  account,
}: KonnectorModalProps) => {
  const { t } = useI18n()
  const fluidName: string = FluidType[fluidType]
  const [index, setIndex] = useState<number>(0)
  const shuffledWaitingTexts = useMemo(() => {
    if (fluidType) {
      return shuffle(connectionWaitingText)
    } else {
      return connectionWaitingText
    }
  }, [fluidType])
  const firstConnectionWaitingTexts = firstConnectionWaitingText.concat(
    ...shuffledWaitingTexts
  )
  const [showCommonErrors, setShowCommonErrors] = useState(false)

  const getUpdatingText = useCallback(() => {
    return (
      <div className="kmodal-waiting-text text-18-italic">
        {shuffledWaitingTexts.map((text, idx) => (
          <div
            key={text.second}
            className={classNames('waiting-text', {
              ['show']: idx === index % shuffledWaitingTexts.length,
            })}
          >
            <p>{text.first}</p>
            <p>{text.second}</p>
          </div>
        ))}
      </div>
    )
  }, [index, shuffledWaitingTexts])

  const getConnectionText = useCallback(() => {
    return (
      <div className="kmodal-waiting-text text-18-italic">
        {isLogging ? (
          <p className="text-18-white">{t('konnector_modal.logging_txt')}</p>
        ) : (
          firstConnectionWaitingTexts.map((text, idx) => (
            <div
              key={text.second}
              className={classNames('waiting-text', {
                ['show']: idx === index % firstConnectionWaitingTexts.length,
              })}
            >
              <p>{text.first}</p>
              <p>{text.second}</p>
            </div>
          ))
        )}
      </div>
    )
  }, [firstConnectionWaitingTexts, index, isLogging, t])

  /** Returns connection success contents, depending on the fluid and update status */
  const connectionSuccessContent = () => (
    <div className="konnector-config">
      <Icon icon={successIcon} size={48} />
      <div className="kcs-picto-txt text-20-bold">
        {t(`konnector_modal.success_${isUpdating ? 'update_' : ''}txt`)}
      </div>
      <b>
        {t(
          `konnector_modal.success_data_${
            isUpdating ? 'update_' : ''
          }${fluidName.toLowerCase()}`
        )}
      </b>
      <p
        style={{ fontWeight: 400 }}
        dangerouslySetInnerHTML={{
          __html: t(
            `konnector_modal.success_data_additional_${
              isUpdating ? 'update_' : ''
            }${fluidName.toLowerCase()}`
          ),
        }}
      />
    </div>
  )

  useEffect(() => {
    const interval = setInterval(() => {
      if (open && !state) {
        setIndex(prev => prev + 1)
      }
    }, 8000)
    return () => {
      clearInterval(interval)
    }
  }, [open, state])

  return (
    <Dialog
      open={open}
      disableEscapeKeyDown
      onClose={(event, reason): void => {
        event &&
          reason !== 'backdropClick' &&
          handleCloseClick(state === SUCCESS_EVENT)
      }}
      aria-labelledby={'accessibility-title'}
      classes={{
        root: 'modal-root',
        paper: 'modal-paper',
      }}
    >
      <div id={'accessibility-title'}>
        {t('konnector_modal.accessibility.window_title')}
      </div>
      <div className="kmodal-content">
        {open && !state ? (
          <>
            <Loader fluidType={fluidType} />
            {!isLogging && (
              <div className="kmodal-content-text kmodal-content-text-center text-16-normal">
                <div className="kc-wait text-16-bold">
                  {t(
                    `konnector_modal.loading_data${isUpdating ? '_update' : ''}`
                  )}
                </div>
              </div>
            )}
            {isUpdating && getUpdatingText()}
            {!isUpdating && getConnectionText()}
          </>
        ) : (
          <>
            <div className="kmodal-info">
              {state === ERROR_EVENT && (
                <>
                  {error === KonnectorError.LOGIN_FAILED && (
                    // LOGIN FAILED FOR ENEDIS AND EGL
                    <div className="konnector-config">
                      <Icon icon={errorIcon} size={48} />
                      <div className="kce-picto-txt text-20-bold">
                        {t('konnector_modal.error_txt')}
                      </div>
                      <div>
                        {t(
                          `konnector_modal.error_credentials_${
                            isUpdating ? 'update_' : ''
                          }${fluidName.toLowerCase()}`
                        )}
                      </div>
                      {fluidType === FluidType.ELECTRICITY && !isUpdating && (
                        <div className="elec-fail">
                          {t(
                            `konnector_modal.error_credentials_${fluidName.toLowerCase()}_2`
                          )}
                        </div>
                      )}
                      {/* Show common errors */}
                      {fluidType === FluidType.ELECTRICITY && (
                        <>
                          {!showCommonErrors && (
                            <span
                              className="commonErrors"
                              onClick={() => setShowCommonErrors(true)}
                            >
                              {t('konnector_modal.show_common_error')}
                            </span>
                          )}
                          {showCommonErrors && (
                            <div
                              dangerouslySetInnerHTML={{
                                __html: t(
                                  'konnector_modal.show_common_error_list'
                                ),
                              }}
                            />
                          )}
                        </>
                      )}
                    </div>
                  )}
                  {error === KonnectorError.TERMS_VERSION_MISMATCH &&
                    isUpdating &&
                    fluidType === FluidType.ELECTRICITY && (
                      // MISMATCH UPDATE ERROR ENEDIS
                      <div className="kce-picto-txt konnector-config mismatch">
                        <Icon icon={EnedisIcon} width={120} height={80} />
                        <div className="title text-20-bold">
                          {t('konnector_modal.mismatch.title')}
                        </div>
                        <div className="text-15-white">
                          {t('konnector_modal.mismatch.text1')}
                        </div>
                        <div className="text-15-white">
                          {t('konnector_modal.mismatch.text2')}
                        </div>
                        <div className="info text-15-normal">
                          {t('konnector_modal.mismatch.text3')}
                        </div>
                      </div>
                    )}
                  {error === KonnectorError.CHALLENGE_ASKED &&
                    fluidType === FluidType.GAS && (
                      // CONSENT FORM ERROR GRDF
                      <div className="konnector-config">
                        <Icon icon={errorIcon} size={48} />
                        <div className="kce-picto-txt text-20-bold">
                          {t('konnector_modal.error_txt')}
                        </div>
                        <div className="title text-20-bold">
                          {t('konnector_modal.error_consent_form_gas_title')}
                        </div>
                        <div className="err-data-2">
                          {t('konnector_modal.error_consent_form_gas_content')}
                        </div>
                        <div className="err-data-2">
                          {t(
                            'konnector_modal.error_consent_form_gas_content_2'
                          )}
                        </div>
                      </div>
                    )}
                  {error !== KonnectorError.LOGIN_FAILED &&
                    error !== KonnectorError.TERMS_VERSION_MISMATCH &&
                    error !== KonnectorError.CHALLENGE_ASKED && (
                      // DEFAULT CASE
                      <div className="konnector-config">
                        {console.log('errortype', error)}
                        <Icon icon={errorIcon} size={48} />
                        <div className="kce-picto-txt text-20-bold">
                          {t('konnector_modal.error_txt')}
                        </div>
                        <div>
                          {t(
                            `konnector_modal.error_data_${
                              isUpdating ? 'update_' : ''
                            }${fluidName.toLowerCase()}`
                          )}
                        </div>
                        <div className="err-data-2">
                          {t('konnector_modal.error_data_2')}
                        </div>
                      </div>
                    )}
                </>
              )}
              {state === SUCCESS_EVENT && connectionSuccessContent()}
              <KonnectorModalFooter
                state={state}
                error={error}
                handleCloseClick={handleCloseClick}
                handleAccountDeletion={handleAccountDeletion}
                account={account}
                isUpdating={isUpdating}
              />
            </div>
          </>
        )}
      </div>
    </Dialog>
  )
}

export default KonnectorModal
