import {
  Accordion,
  AccordionDetails,
  AccordionSummary,
  Button,
} from '@material-ui/core'
import chevronDown from 'assets/icons/ico/chevron-down.svg'
import ErrorNotif from 'assets/icons/ico/notif_error.svg'
import PartnersIssueNotif from 'assets/icons/ico/notif_maintenance.svg'
import OfflinePicto from 'assets/icons/visu/offline-param.svg'
import classNames from 'classnames'
import StyledIcon from 'components/CommonKit/Icon/StyledIcon'
import Connection from 'components/Connection/Connection'
import ConnectionNotFound from 'components/Connection/ConnectionNotFound'
import ConnectionResult from 'components/Connection/ConnectionResult'
import KonnectorModal from 'components/Konnector/KonnectorModal'
import { useClient } from 'cozy-client'
import { isKonnectorRunning } from 'cozy-harvest-lib/dist/helpers/triggers'
import ConnectionFlow from 'cozy-harvest-lib/dist/models/ConnectionFlow'
import {
  ERROR_EVENT,
  LOGIN_SUCCESS_EVENT,
  SUCCESS_EVENT,
} from 'cozy-harvest-lib/dist/models/flowEvents'
import { useI18n } from 'cozy-ui/transpiled/react/I18n'
import Icon from 'cozy-ui/transpiled/react/Icon'
import { FluidState, FluidType } from 'enum/fluid.enum'
import { FluidSlugType } from 'enum/fluidSlug.enum'
import { KonnectorError } from 'enum/konnectorError.enum'
import { UsageEventType } from 'enum/usageEvent.enum'
import { UserChallengeState } from 'enum/userChallenge.enum'
import { UserDuelState } from 'enum/userDuel.enum'
import { DateTime } from 'luxon'
import {
  Account,
  AccountAuthData,
  FluidConnection,
  FluidStatus,
  Konnector,
  Trigger,
  UsageEvent,
} from 'models'
import { PartnersInfo } from 'models/partnersInfo.model'
import React, {
  Dispatch,
  useCallback,
  useEffect,
  useMemo,
  useState,
} from 'react'
import { useDispatch, useSelector } from 'react-redux'
import { useNavigate } from 'react-router-dom'
import AccountService from 'services/account.service'
import ChallengeService from 'services/challenge.service'
import DateChartService from 'services/dateChart.service'
import FluidService from 'services/fluid.service'
import PartnersInfoService from 'services/partnersInfo.service'
import UsageEventService from 'services/usageEvent.service'
import { AppActionsTypes, AppStore } from 'store'
import { setChallengeConsumption } from 'store/challenge/challenge.actions'
import { setSelectedDate, setShowOfflineData } from 'store/chart/chart.slice'
import {
  setFluidStatus,
  toggleChallengeDuelNotification,
  updatedFluidConnection,
} from 'store/global/global.actions'
import { openConnectionModal } from 'store/modal/modal.slice'
import { getAddPicto, getParamPicto } from 'utils/picto'
import { getKonnectorSlug } from 'utils/utils'
import './konnectorViewerCard.scss'

interface KonnectorViewerCardProps {
  fluidStatus: FluidStatus
  isParam: boolean
  isDisconnected: boolean
  showOfflineData: boolean
  active: boolean
  fluidType: FluidType
  setActive: React.Dispatch<React.SetStateAction<boolean>>
}

const KonnectorViewerCard: React.FC<KonnectorViewerCardProps> = ({
  fluidStatus,
  isParam,
  isDisconnected,
  showOfflineData,
  active,
  fluidType,
  setActive,
}: KonnectorViewerCardProps) => {
  const { t } = useI18n()
  const client = useClient()
  const dispatch = useDispatch<Dispatch<AppActionsTypes>>()
  const navigate = useNavigate()

  const fluidSlug = fluidStatus.connection.konnectorConfig.slug
  const fluidState = fluidStatus.status
  const konnector: Konnector | null = fluidStatus.connection.konnector
  const account: Account | null = fluidStatus.connection.account
  const trigger: Trigger | null = fluidStatus.connection.trigger
  const {
    challenge: { currentChallenge },
    global: { fluidStatus: statusArray, shouldRefreshConsent, partnersInfo },
  } = useSelector((state: AppStore) => state.ecolyo)

  const [openModal, setOpenModal] = useState(false)
  const [isUpdating, setIsUpdating] = useState(false)
  const [isLogging, setIsLogging] = useState(
    fluidSlug !== FluidSlugType.GAS ? true : false
  )
  const [konnectorErrorDescription, setKonnectorErrorDescription] =
    useState<KonnectorError | null>(null)
  const [konnectorState, setKonnectorState] = useState<string | null>(null)
  const [updatedFluidStatus, setUpdatedFluidStatus] = useState<FluidStatus[]>(
    []
  )
  const [isOutdatedData, setIsOutdatedData] = useState<number | null>(null)
  const fluidService = useMemo(() => new FluidService(client), [client])
  const partnersInfoService = useMemo(
    () => new PartnersInfoService(client),
    [client]
  )

  const lastDataDate =
    fluidType !== FluidType.MULTIFLUID && statusArray[fluidType].lastDataDate
      ? statusArray[fluidType].lastDataDate!.toLocaleString() + fluidType
      : fluidType

  const iconType = getParamPicto(fluidStatus.fluidType)
  const iconAddType = isParam
    ? getParamPicto(fluidStatus.fluidType)
    : getAddPicto(fluidStatus.fluidType)

  const toggleAccordion = () => {
    setActive(prev => !prev)
  }

  const updateGlobalFluidStatus = useCallback(async (): Promise<
    FluidStatus[]
  > => {
    const _updatedFluidStatus: FluidStatus[] =
      await fluidService.getFluidStatus()
    setUpdatedFluidStatus(_updatedFluidStatus)
    const refDate = DateTime.fromISO('0001-01-01')
    let _lastDataDate: DateTime | null = DateTime.fromISO('0001-01-01')
    for (const fluid of _updatedFluidStatus) {
      if (fluid?.lastDataDate && fluid?.lastDataDate > _lastDataDate) {
        _lastDataDate = fluid.lastDataDate
      }
    }
    if (_lastDataDate > refDate) {
      dispatch(setSelectedDate(_lastDataDate))
    }

    return _updatedFluidStatus
  }, [dispatch, fluidService])

  const refreshChallengeState = useCallback(async () => {
    if (
      currentChallenge?.state === UserChallengeState.DUEL &&
      currentChallenge?.duel.state === UserDuelState.ONGOING
    ) {
      const challengeService = new ChallengeService(client)
      const { updatedUserChallenge, dataloads } =
        await challengeService.initChallengeDuelProgress(currentChallenge)
      dispatch(setChallengeConsumption(updatedUserChallenge, dataloads))
      // Check is duel is done and display notification
      const { isDone } = await challengeService.isChallengeDone(
        updatedUserChallenge,
        dataloads
      )
      dispatch(toggleChallengeDuelNotification(isDone))
    }
  }, [client, dispatch, currentChallenge])

  const handleAccountDeletion = useCallback(async () => {
    await refreshChallengeState()
    const _updatedFluidStatus = await updateGlobalFluidStatus()
    if (_updatedFluidStatus.length > 0) {
      const partnersInfo: PartnersInfo | undefined =
        await partnersInfoService.getPartnersInfo()
      const updatedFluidStatus: FluidStatus[] =
        await fluidService.getFluidStatus(partnersInfo)
      dispatch(setFluidStatus(updatedFluidStatus))
    }
    setActive(false)
  }, [
    refreshChallengeState,
    updateGlobalFluidStatus,
    setActive,
    partnersInfoService,
    fluidService,
    dispatch,
  ])

  const handleConnectionEnd = useCallback(
    async (isSuccess?: boolean) => {
      // CASE FOR GLOBAL LOGIN FAILED
      const isGlobalLoginFailed =
        konnectorErrorDescription === KonnectorError.LOGIN_FAILED ||
        konnectorErrorDescription === KonnectorError.UNKNOWN_ERROR ||
        konnectorErrorDescription === KonnectorError.CHALLENGE_ASKED ||
        konnectorErrorDescription === KonnectorError.CRITICAL

      // CASE FOR ENEDIS CODE INSEE ERROR
      const isEnedisCodeInseeError =
        !isUpdating && fluidType === FluidType.ELECTRICITY

      const shouldDeleteAccount =
        account &&
        !isSuccess &&
        fluidStatus &&
        fluidStatus.connection.account &&
        (isGlobalLoginFailed || isEnedisCodeInseeError)

      if (shouldDeleteAccount) {
        // KEEP CREDENTIALS FOR EGL
        if (
          fluidSlug === FluidSlugType.WATER &&
          fluidStatus.connection.account?.auth
        ) {
          const auth = fluidStatus.connection.account.auth as AccountAuthData
          fluidStatus.connection.konnectorConfig.lastKnownCredentials =
            auth.login
        }
        // DELETE ACCOUNT
        const accountService = new AccountService(client)
        await accountService.deleteAccount(account)
        await handleAccountDeletion()
      } else {
        if (isSuccess && fluidStatus.lastDataDate === null) {
          // UPDATE THE DACC EVENT STATUS TO SUCCESS
          await UsageEventService.udpateConnectionAttemptEvent(
            client,
            fluidSlug
          )
        }
        if (updatedFluidStatus.length > 0) {
          // const partnersInfo = await partnersInfoService.getPartnersInfo()
          const updatedFluidStatus = await fluidService.getFluidStatus(
            partnersInfo
          )
          dispatch(setFluidStatus(updatedFluidStatus))
        }
      }

      setActive(false)
      setOpenModal(false)
      // TODO null state seems to be read before modal closing and display a success icon in modal
      setKonnectorState(null)
      setKonnectorErrorDescription(null)
    },
    [
      account,
      konnectorErrorDescription,
      fluidStatus,
      isUpdating,
      fluidType,
      setActive,
      fluidSlug,
      client,
      handleAccountDeletion,
      updatedFluidStatus.length,
      fluidService,
      partnersInfo,
      dispatch,
    ]
  )

  const sendUsageEventSuccess = useCallback(
    async (
      konnectorSlug: string,
      isConnexion: boolean
    ): Promise<UsageEvent> => {
      return UsageEventService.addEvent(client, {
        type: isConnexion
          ? UsageEventType.KONNECTOR_CONNECT_EVENT
          : UsageEventType.KONNECTOR_REFRESH_EVENT,
        target: konnectorSlug,
        result: 'success',
      })
    },
    [client]
  )

  const sendUsageEventError = useCallback(
    async (
      konnectorSlug: string,
      isConnexion: boolean
    ): Promise<UsageEvent> => {
      return UsageEventService.addEvent(client, {
        type: isConnexion
          ? UsageEventType.KONNECTOR_CONNECT_EVENT
          : UsageEventType.KONNECTOR_REFRESH_EVENT,
        target: konnectorSlug,
        result: 'error',
      })
    },
    [client]
  )

  const toggleModalConnection = () => {
    switch (fluidType) {
      case FluidType.ELECTRICITY:
        navigate('/sge-connect')
        break
      case FluidType.GAS:
      case FluidType.WATER:
        dispatch(setShowOfflineData(false))
        dispatch(openConnectionModal(true))
        break
    }
  }

  const getConnectionCard = useCallback(() => {
    if (showOfflineData && !account) {
      return (
        <Button
          classes={{
            root: 'btn-highlight',
            label: 'text-16-bold',
          }}
          onClick={toggleModalConnection}
        >
          {t(`auth.${getKonnectorSlug(fluidType)}.connect`)}
        </Button>
      )
    }
    if (fluidState === FluidState.KONNECTOR_NOT_FOUND && !isUpdating) {
      return <ConnectionNotFound konnectorSlug={fluidSlug} />
    }
    // Handle login failed for EGL
    else if (
      (fluidState === FluidState.ERROR_LOGIN_FAILED &&
        fluidType === FluidType.WATER) ||
      (account && fluidStatus.status !== FluidState.NOT_CONNECTED)
    ) {
      return (
        <ConnectionResult
          fluidStatus={fluidStatus}
          handleAccountDeletion={handleAccountDeletion}
          fluidType={fluidType}
          key={lastDataDate}
        />
      )
    } else {
      return <Connection fluidStatus={fluidStatus} />
    }
  }, [
    fluidState,
    isUpdating,
    account,
    fluidStatus,
    fluidSlug,
    handleAccountDeletion,
    fluidType,
    lastDataDate,
  ])

  const callbackResponse = useCallback(
    async (_state: string) => {
      if (_state !== LOGIN_SUCCESS_EVENT) {
        const updatedConnection: FluidConnection = {
          ...fluidStatus.connection,
          shouldLaunchKonnector: false,
        }
        dispatch(
          updatedFluidConnection(fluidStatus.fluidType, updatedConnection)
        )
        Promise.all([
          await refreshChallengeState(),
          await updateGlobalFluidStatus(),
        ])
        setKonnectorState(_state)
      }
    },
    [
      dispatch,
      fluidStatus.connection,
      fluidStatus.fluidType,
      refreshChallengeState,
      updateGlobalFluidStatus,
    ]
  )
  const displayKonnectorIcon = useCallback(() => {
    return (
      <div className="konnector-icon">
        <Icon
          icon={fluidStatus.connection.account ? iconType : OfflinePicto}
          size={49}
        />
        {fluidStatus.maintenance ? (
          <StyledIcon
            icon={PartnersIssueNotif}
            size={24}
            className="konnector-state-picto"
          />
        ) : (
          (fluidStatus.status === FluidState.ERROR ||
            isOutdatedData ||
            fluidStatus.status === FluidState.ERROR_LOGIN_FAILED) &&
          fluidStatus.connection.account && (
            <StyledIcon
              icon={ErrorNotif}
              size={24}
              className="konnector-state-picto"
            />
          )
        )}
      </div>
    )
  }, [
    fluidStatus.connection.account,
    fluidStatus.maintenance,
    fluidStatus.status,
    iconAddType,
    iconType,
    isOutdatedData,
  ])

  const displayKonnectorHeader = useCallback(() => {
    if (fluidStatus.maintenance) {
      return (
        <span className="text-16-bold">
          {t(`konnector_options.partner_issue`)}
        </span>
      )
    } else if (isOutdatedData && fluidStatus.connection.account) {
      return (
        <span className="text-16-bold outdated">
          {t('konnector_options.outdated', {
            isOutdatedData: isOutdatedData,
          })}
        </span>
      )
    } else if (fluidStatus.connection.account && !isOutdatedData) {
      return t(`FLUID.${FluidType[fluidStatus.fluidType]}.LABEL`)
    } else {
      return t(
        `konnector_options.label_offline_${FluidType[
          fluidStatus.fluidType
        ].toLowerCase()}`
      )
    }
  }, [
    fluidStatus.connection.account,
    fluidStatus.fluidType,
    fluidStatus.maintenance,
    isOutdatedData,
    t,
  ])

  // If user has selected accept button on Expired consent modal, then delete its account
  useEffect(() => {
    async function deleteAccountForConsentRefresh() {
      if (shouldRefreshConsent && account) {
        const accountService = new AccountService(client)
        await accountService.deleteAccount(account)
        await handleAccountDeletion()
      }
    }
    deleteAccountForConsentRefresh()
  }, [account, client, handleAccountDeletion, shouldRefreshConsent])

  useEffect(() => {
    let subscribed = true
    async function getData() {
      if (
        fluidStatus.connection.shouldLaunchKonnector &&
        !isKonnectorRunning(trigger)
      ) {
        if (subscribed) {
          if (fluidStatus.connection.isUpdating) setIsUpdating(true)
          setOpenModal(true)
          fluidStatus.connection.shouldLaunchKonnector = false
        }

        const connectionFlow = new ConnectionFlow(client, trigger, konnector)
        await connectionFlow.launch()
        connectionFlow.jobWatcher.on(ERROR_EVENT, () => {
          sendUsageEventError(fluidSlug, fluidStatus.lastDataDate === null)
          setKonnectorErrorDescription(connectionFlow.jobWatcher.on()._error)
          callbackResponse(ERROR_EVENT)
        })
        connectionFlow.jobWatcher.on(LOGIN_SUCCESS_EVENT, () => {
          setIsLogging(false)
          sendUsageEventSuccess(fluidSlug, fluidStatus.lastDataDate === null)
        })
        connectionFlow.jobWatcher.on(SUCCESS_EVENT, () => {
          sendUsageEventSuccess(fluidSlug, fluidStatus.lastDataDate === null)
          callbackResponse(SUCCESS_EVENT)
        })
      }
    }
    !shouldRefreshConsent && getData()
    const dateChartService = new DateChartService()
    setIsOutdatedData(
      dateChartService.isDataOutdated(fluidStatus.lastDataDate, fluidType)
    )
    return () => {
      subscribed = false
    }
  }, [
    client,
    konnector,
    trigger,
    fluidStatus.connection,
    fluidStatus.connection.shouldLaunchKonnector,
    fluidStatus.connection.isUpdating,
    fluidStatus.fluidType,
    callbackResponse,
    setActive,
    fluidStatus.lastDataDate,
    fluidType,
    sendUsageEventError,
    fluidSlug,
    sendUsageEventSuccess,
    shouldRefreshConsent,
  ])

  return (
    <>
      {!isDisconnected ? (
        <Accordion
          expanded={active}
          onChange={toggleAccordion}
          classes={{
            root: `expansion-panel-root ${
              !fluidStatus.maintenance &&
              (fluidStatus.status === FluidState.ERROR ||
                fluidStatus.status === FluidState.ERROR_LOGIN_FAILED)
                ? 'red-border'
                : ''
            }`,
          }}
        >
          <AccordionSummary
            aria-label={t(
              `konnector_options.accessibility.button_toggle_detail_${FluidType[
                fluidStatus.fluidType
              ].toLowerCase()}`
            )}
            expandIcon={
              <Icon icon={chevronDown} size={16} className="accordion-icon" />
            }
            classes={{
              root: 'expansion-panel-summary',
              content: 'expansion-panel-content',
            }}
          >
            {displayKonnectorIcon()}
            <div
              className={classNames('text-16-bold konnector-title', {
                [`${FluidType[fluidStatus.fluidType].toLowerCase()}-connected`]:
                  fluidStatus.status !== FluidState.NOT_CONNECTED &&
                  !fluidStatus.maintenance,
              })}
            >
              {displayKonnectorHeader()}
            </div>
          </AccordionSummary>
          <AccordionDetails
            classes={{
              root: 'expansion-panel-details',
            }}
          >
            {getConnectionCard()}
          </AccordionDetails>
        </Accordion>
      ) : (
        <>
          <AccordionDetails>{getConnectionCard()}</AccordionDetails>
        </>
      )}
      <KonnectorModal
        open={openModal}
        isUpdating={isUpdating}
        isLogging={isLogging}
        state={konnectorState}
        error={konnectorErrorDescription}
        fluidType={fluidStatus.fluidType}
        handleCloseClick={handleConnectionEnd}
        handleAccountDeletion={handleAccountDeletion}
        account={account}
      />
    </>
  )
}
export default KonnectorViewerCard
