import { Button } from '@material-ui/core'
import { useClient } from 'cozy-client'
import { useI18n } from 'cozy-ui/transpiled/react/I18n'
import { FluidType } from 'enum/fluid.enum'
import { TimeStep } from 'enum/timeStep.enum'
import { UsageEventType } from 'enum/usageEvent.enum'
import { DateTime } from 'luxon'
import React, { Dispatch } from 'react'
import { useDispatch, useSelector } from 'react-redux'
import DateChartService from 'services/dateChart.service'
import UsageEventService from 'services/usageEvent.service'
import { AppActionsTypes, AppStore } from 'store'
import {
  setCurrentIndex,
  setCurrentTimeStep,
  setSelectedDate,
} from 'store/chart/chart.slice'
import './timeStepSelector.scss'

interface TimeStepSelectorProps {
  fluidType: FluidType
}

const TimeStepSelector: React.FC<TimeStepSelectorProps> = ({
  fluidType,
}: TimeStepSelectorProps) => {
  const { currentTimeStep, selectedDate } = useSelector(
    (state: AppStore) => state.ecolyo.chart
  )
  const { t } = useI18n()
  const dispatch = useDispatch<Dispatch<AppActionsTypes>>()
  const dateChartService = new DateChartService()
  const client = useClient()
  const timeStepElecArray: TimeStep[] = [
    TimeStep.HALF_AN_HOUR,
    TimeStep.WEEK,
    TimeStep.DAY,
    TimeStep.MONTH,
    TimeStep.YEAR,
  ]
  const timeStepMultiArray: TimeStep[] = [
    TimeStep.WEEK,
    TimeStep.DAY,
    TimeStep.MONTH,
    TimeStep.YEAR,
  ]
  const timeStepArray: TimeStep[] =
    fluidType === FluidType.ELECTRICITY
      ? [...timeStepElecArray]
      : [...timeStepMultiArray]

  const handleToday = () => {
    const today = DateTime.local()
      .setZone('utc', {
        keepLocalTime: true,
      })
      .startOf('day')
    UsageEventService.addEvent(client, {
      type: UsageEventType.CONSUMPTION_CHANGE_TIMESTEP_EVENT,
      target: TimeStep[TimeStep.WEEK].toLowerCase(),
      context: FluidType[fluidType].toLowerCase(),
    })
    dispatch(setCurrentIndex(0))
    dispatch(setSelectedDate(today))
    dispatch(setCurrentTimeStep(TimeStep.WEEK))
  }
  const handleChangeTimeStep = (targetTimestep: TimeStep) => {
    const index: number = dateChartService.defineDateIndex(
      targetTimestep,
      selectedDate
    )
    UsageEventService.addEvent(client, {
      type: UsageEventType.CONSUMPTION_CHANGE_TIMESTEP_EVENT,
      target: TimeStep[currentTimeStep].toLowerCase(),
      context: FluidType[fluidType].toLowerCase(),
    })
    dispatch(setCurrentTimeStep(targetTimestep))
    dispatch(setCurrentIndex(index))
  }
  return (
    <div className={'timestep-selector'}>
      <Button
        onClick={handleToday}
        classes={{
          root: 'btn-today',
          label: 'text-13-normal',
        }}
      >
        {t('timestep.today')}
      </Button>
      <div className={'timestep-container'}>
        <ul
          className={`timestep-bar ${
            fluidType === FluidType.ELECTRICITY && 'elec-bar'
          }`}
        >
          {timeStepArray.map(step => (
            <React.Fragment key={step}>
              <li
                className={
                  step === currentTimeStep ? 'active circle' : 'circle'
                }
                onClick={() => handleChangeTimeStep(step)}
                id={TimeStep[step].toLowerCase()}
              >
                <span className="clickable-area" />
                <span className={'text text-14-normal'}>
                  {t(`timestep.${TimeStep[step].toLowerCase()}.period`)}
                </span>
              </li>
              <li className="bar" />
            </React.Fragment>
          ))}
        </ul>
      </div>
    </div>
  )
}

export default TimeStepSelector
