import { Client, Q, QueryDefinition, QueryResult } from 'cozy-client'
import { Doctype } from 'cozy-client/types/types'
import { ENEDIS_MINUTE_DOCTYPE } from 'doctypes'
import { DataloadState } from 'enum/dataload.enum'
import { FluidType } from 'enum/fluid.enum'
import { TimeStep } from 'enum/timeStep.enum'
import { DateTime } from 'luxon'
import {
  Datachart,
  Dataload,
  DataloadEntity,
  DataloadValueDetail,
  FluidStatus,
  PerformanceIndicator,
  TimePeriod,
} from 'models'
import { EnedisMonthlyAnalysisData } from 'models/enedisMonthlyAnalysis'
import ConsumptionFormatterService from 'services/consumptionFormatter.service'
import ConsumptionValidatorService from 'services/consumptionValidator.service'
import ConverterService from 'services/converter.service'
import QueryRunnerService from 'services/queryRunner.service'

export interface ISingleFluidChartData {
  chartData: Datachart | null
  chartFluid: FluidType
}

export default class ConsumptionDataManager {
  private readonly _client: Client
  private readonly _consumptionFormatterService: ConsumptionFormatterService
  private readonly _queryRunnerService: QueryRunnerService
  private readonly _consumptionValidatorService: ConsumptionValidatorService

  constructor(_client: Client) {
    this._client = _client
    this._consumptionFormatterService = new ConsumptionFormatterService()
    this._queryRunnerService = new QueryRunnerService(this._client)
    this._consumptionValidatorService = new ConsumptionValidatorService()
  }

  /**
   * Get graph data according on timeStep and fluidType
   * @param timePeriod TimePeriod
   * @param timeStep TimeStep
   * @param fluidTypes FluidType[]
   * @param fluidStatus FluidStatus[]
   * @param compareTimePeriod - Optional TimePeriod
   * @param isHome - Optional boolean
   * @param isExport - Optional boolean
   * @returns DataChart | null
   */
  public async getGraphData(
    timePeriod: TimePeriod,
    timeStep: TimeStep,
    fluidTypes: FluidType[],
    fluidStatus?: FluidStatus[],
    compareTimePeriod?: TimePeriod,
    isHome?: boolean,
    isExport?: boolean
  ): Promise<Datachart | null> {
    const InputisValid: boolean =
      this._consumptionValidatorService.ValidateGetGraphData(
        timePeriod,
        timeStep,
        fluidTypes,
        compareTimePeriod,
        isExport
      )
    if (!InputisValid) return null
    if (fluidTypes.length === 1 && !isHome) {
      const fluidType: FluidType = fluidTypes[0]
      // running the query
      const fetchedData: Datachart | null =
        await this.fetchSingleFluidGraphData(
          timePeriod,
          timeStep,
          fluidType,
          compareTimePeriod
        )

      // formatting data
      const formattedData: Datachart | null = this.formatGraphDataManager(
        fetchedData,
        timeStep,
        timePeriod,
        compareTimePeriod || null,
        fluidType,
        fluidStatus ? fluidStatus[fluidType] : undefined
      )
      return formattedData
    } else if (fluidTypes.length > 1 || isHome) {
      const toBeAggregatedData: ISingleFluidChartData[] = []
      for (const fluidType of fluidTypes) {
        const fetchedData = await this.fetchSingleFluidGraphData(
          timePeriod,
          timeStep,
          fluidType,
          compareTimePeriod
        )
        // formatting data
        const formattedData = this.formatGraphDataManager(
          fetchedData,
          timeStep,
          timePeriod,
          compareTimePeriod || null,
          fluidType,
          fluidStatus ? fluidStatus[fluidType] : undefined
        )
        // validating output data
        toBeAggregatedData.push({
          chartData: formattedData,
          chartFluid: fluidType,
        })
      }
      const aggregatedData: Datachart | null =
        this.aggregateGraphData(toBeAggregatedData)
      return aggregatedData
    } else return null
  }

  public async getMaxLoad(
    maxTimePeriod: TimePeriod,
    timeStep: TimeStep,
    fluidTypes: FluidType[],
    compareMaxTimePeriod?: TimePeriod,
    isHome?: boolean,
    withDate?: boolean
  ): Promise<number | null | Dataload> {
    let allData
    if (isHome) {
      allData = await this.getGraphData(
        maxTimePeriod,
        timeStep,
        fluidTypes,
        undefined,
        compareMaxTimePeriod,
        isHome
      )
      return allData?.actualData
        ? Math.max(...allData.actualData.map(d => d.value))
        : 0
    } else {
      const max = await this._queryRunnerService.fetchFluidMaxData(
        maxTimePeriod,
        timeStep,
        fluidTypes[0],
        withDate
      )
      return max
    }
  }

  // fetch last dataload available for a given fluid - return the daily data
  public async getLastDataload(
    fluidTypes: FluidType
  ): Promise<Dataload[] | null> {
    const timePeriod = {
      startDate: DateTime.now().plus({ days: -3 }).startOf('day'),
      endDate: DateTime.now(),
    }

    const data = await this._queryRunnerService.fetchFluidData(
      timePeriod,
      TimeStep.DAY,
      fluidTypes
    )
    return data
  }

  public async getPerformanceIndicators(
    timePeriod: TimePeriod,
    timeStep: TimeStep,
    fluidTypes: FluidType[],
    compareTimePeriod?: TimePeriod
  ): Promise<PerformanceIndicator[]> {
    const performanceIndicators: PerformanceIndicator[] = []
    for (const fluidType of fluidTypes) {
      const graphData: Datachart | null = await this.getGraphData(
        timePeriod,
        timeStep,
        [fluidType],
        undefined,
        compareTimePeriod
      )

      if (graphData) {
        const performanceIndicator: PerformanceIndicator = {
          value: null,
          compareValue: null,
          percentageVariation: null,
          price: null,
        }

        const actualDataIsValid = this.PerformanceIndicatorsDataIsValid(
          graphData.actualData
        )

        if (actualDataIsValid) {
          performanceIndicator.value = this.calculatePerformanceIndicatorValue(
            graphData.actualData
          )
          if (graphData.actualData[0].price)
            performanceIndicator.price =
              this.calculatePerformanceIndicatorPrice(graphData.actualData)
        }

        if (
          actualDataIsValid &&
          graphData.comparisonData &&
          this.PerformanceIndicatorsDataIsValid(graphData.comparisonData)
        ) {
          const comparisonSumValue = this.calculatePerformanceIndicatorValue(
            graphData.comparisonData
          )
          performanceIndicator.compareValue = comparisonSumValue
          performanceIndicator.percentageVariation =
            this.calculatePerformanceIndicatorVariationPercentage(
              performanceIndicator.value || 0,
              comparisonSumValue
            )
        }

        performanceIndicators[fluidType] = performanceIndicator
      }
    }

    return performanceIndicators
  }

  private PerformanceIndicatorsDataIsValid(data: Dataload[]): boolean {
    if (!data) return false

    const missingValue = data.find(element => element.value === -1)
    if (missingValue) return false

    return true
  }

  public calculatePerformanceIndicatorValue(data: Dataload[]): number {
    return data.reduce((a, b) => (b.value !== -1 ? a + b.value : a), 0)
  }

  public calculatePerformanceIndicatorPrice(data: Dataload[]): number {
    return data.reduce((a, b) => (b.price ? a + b.price : a), 0)
  }

  private calculatePerformanceIndicatorVariationPercentage(
    dataSum: number,
    comparisonDataSum: number
  ): number | null {
    return comparisonDataSum !== 0 ? dataSum / comparisonDataSum - 1 : null
  }

  private async fetchSingleFluidGraphData(
    timePeriod: TimePeriod,
    timeStep: TimeStep,
    fluidType: FluidType,
    compareTimePeriod?: TimePeriod
  ): Promise<Datachart | null> {
    let actualData: Dataload[] | null = []
    let comparisonData: Dataload[] | null = []
    let singleFluidGraphData: Datachart | null = null

    if (compareTimePeriod) {
      const result = await Promise.all([
        this._queryRunnerService.fetchFluidData(
          timePeriod,
          timeStep,
          fluidType
        ),
        this._queryRunnerService.fetchFluidData(
          compareTimePeriod,
          timeStep,
          fluidType
        ),
      ])
      actualData = result[0]
      comparisonData = result[1]
    } else {
      actualData = await this._queryRunnerService.fetchFluidData(
        timePeriod,
        timeStep,
        fluidType
      )
    }

    if (actualData) {
      singleFluidGraphData = {
        actualData: actualData,
        comparisonData: comparisonData,
      }
    }
    return singleFluidGraphData
  }

  private formatGraphDataManager(
    data: Datachart | null,
    timeStep: TimeStep,
    timePeriod: TimePeriod,
    compareTimePeriod: TimePeriod | null,
    fluidType: FluidType,
    fluidStatus?: FluidStatus
  ): Datachart | null {
    if (!data) return null

    const formattedActualData: Dataload[] =
      this._consumptionFormatterService.formatGraphData(
        data.actualData,
        timePeriod,
        timeStep,
        fluidType,
        fluidStatus
      )

    let formattedComparisonData: Dataload[] | null = null
    if (compareTimePeriod)
      formattedComparisonData =
        this._consumptionFormatterService.formatGraphData(
          data.comparisonData ? data.comparisonData : [],
          compareTimePeriod,
          timeStep,
          fluidType,
          fluidStatus
        )

    const result: Datachart = {
      actualData: formattedActualData,
      comparisonData: formattedComparisonData,
    }

    return result
  }

  public async getExportableFluids(
    fluidTypes: FluidType[],
    timeStep: TimeStep
  ): Promise<FluidType[]> {
    const exportableFluids: FluidType[] = []
    for (const fluidType of fluidTypes) {
      if (await this.checkDoctypeEntries(fluidType, timeStep)) {
        exportableFluids.push(fluidType)
      }
    }
    return exportableFluids
  }

  public async fetchAllFirstDateData(
    fluidTypes: FluidType[],
    timeStep?: TimeStep
  ): Promise<(DateTime | null)[]> {
    let firstDay = null
    const firstDays = []
    for (const fluidType of fluidTypes) {
      firstDay =
        (await this._queryRunnerService.getFirstDateData(
          fluidType,
          timeStep
        )) || null
      firstDays.push(firstDay)
    }
    return firstDays
  }

  public async fetchAllLastDateData(
    fluidTypes: FluidType[],
    timeStep?: TimeStep
  ): Promise<(DateTime | null)[]> {
    let lastDay = null
    const lastDays = []
    if (fluidTypes.length === 1) {
      lastDay =
        (await this._queryRunnerService.getLastDateData(
          fluidTypes[0],
          timeStep
        )) || null
      lastDays.push(lastDay)
    } else if (fluidTypes.length > 1) {
      for (const fluidType of fluidTypes) {
        lastDay =
          (await this._queryRunnerService.getLastDateData(
            fluidType,
            timeStep
          )) || null
        lastDays.push(lastDay)
      }
    }
    return lastDays
  }

  public async checkDoctypeEntries(
    fluideType: FluidType,
    timeStep: TimeStep
  ): Promise<boolean> {
    const queryResult = await this._queryRunnerService.getEntries(
      fluideType,
      timeStep
    )
    if (queryResult?.data.length > 0) {
      return true
    }
    return false
  }

  private aggregateGraphData(
    singleFluidCharts: ISingleFluidChartData[]
    // ,withComparison: boolean = true
  ): Datachart | null {
    if (singleFluidCharts[0]?.chartData) {
      const converterService = new ConverterService()
      const resultChartData: Datachart = {
        actualData: [],
        comparisonData: [],
      }

      let length = 0
      if (singleFluidCharts[0].chartData.comparisonData) {
        length = Math.max(
          singleFluidCharts[0].chartData.comparisonData.length,
          singleFluidCharts[0].chartData.actualData.length
        )
      } else length = singleFluidCharts[0].chartData.actualData.length

      for (let i = 0; i < length; i++) {
        let aggregatedConvertedValue = 0
        let comparisonAggregatedConvertedValue = 0

        const tempAggregatedState: DataloadState[] = []
        const tempComparisonAggregatedState: DataloadState[] = []

        let noDataCount = 0
        let comparisonNoDataCount = 0

        const convertedValueDetail: DataloadValueDetail[] = []
        const comparisonConvertedValueDetail: DataloadValueDetail[] = []

        for (const singleFluidChart of singleFluidCharts) {
          if (!singleFluidChart.chartData) break
          if (!singleFluidChart.chartData.actualData[i]) break
          const value = singleFluidChart.chartData.actualData[i].value
          tempAggregatedState.push(
            singleFluidChart.chartData.actualData[i].state
          )

          let convertedValue = -1
          if (value === -1) noDataCount++
          else {
            convertedValue = converterService.LoadToEuro(
              value,
              singleFluidChart.chartFluid,
              singleFluidChart.chartData.actualData[i].price
            )
            aggregatedConvertedValue += convertedValue
          }

          convertedValueDetail[singleFluidChart.chartFluid] = {
            value: convertedValue,
            state: singleFluidChart.chartData.actualData[i].state,
          }

          if (singleFluidChart.chartData.comparisonData?.[i]) {
            const comparisonValue =
              singleFluidChart.chartData.comparisonData[i].value
            tempComparisonAggregatedState.push(
              singleFluidChart.chartData.comparisonData[i].state
            )

            let convertedComparisonValue = -1
            if (comparisonValue === -1) comparisonNoDataCount++
            else {
              convertedComparisonValue = converterService.LoadToEuro(
                comparisonValue,
                singleFluidChart.chartFluid,
                singleFluidChart.chartData.comparisonData[i].price
              )
              comparisonAggregatedConvertedValue += convertedComparisonValue
            }

            comparisonConvertedValueDetail[singleFluidChart.chartFluid] = {
              value: convertedComparisonValue,
              state: singleFluidChart.chartData.comparisonData[i].state,
            }
          }
        }

        if (singleFluidCharts.length === noDataCount)
          aggregatedConvertedValue = -1
        if (singleFluidCharts.length === comparisonNoDataCount)
          comparisonAggregatedConvertedValue = -1

        if (singleFluidCharts[0].chartData.actualData[i]) {
          // Define the aggregated state
          const aggregatedDataloadState: DataloadState =
            this._consumptionFormatterService.defineAggregatedDataloadState(
              tempAggregatedState
            )
          const actualDataLoad: Dataload = {
            date: singleFluidCharts[0].chartData.actualData[i].date,
            value: aggregatedConvertedValue,
            state: aggregatedDataloadState,
            valueDetail:
              aggregatedConvertedValue === -1 ? null : convertedValueDetail,
          }
          resultChartData.actualData.push(actualDataLoad)
        }

        if (
          singleFluidCharts[0].chartData.comparisonData &&
          resultChartData.comparisonData &&
          singleFluidCharts[0].chartData.comparisonData[i]
        ) {
          // Define the aggregated state
          const aggregatedComparisonDataloadState: DataloadState =
            this._consumptionFormatterService.defineAggregatedDataloadState(
              tempComparisonAggregatedState
            )
          const comparisonDataLoad: Dataload = {
            date: singleFluidCharts[0].chartData.comparisonData[i].date,
            value: comparisonAggregatedConvertedValue,
            state: aggregatedComparisonDataloadState,
            valueDetail:
              comparisonAggregatedConvertedValue === -1
                ? null
                : comparisonConvertedValueDetail,
          }
          resultChartData.comparisonData.push(comparisonDataLoad)
        }
      }
      return resultChartData
    }

    return null
  }

  /**
   * getLastHourData
   * @param {Client} client
   * @param {number} month number
   * @returns {Promise<DataloadEntity[]>} usageEvent added
   */
  public async getLastHourData(
    client: Client,
    month: number
  ): Promise<DataloadEntity[]> {
    const query: QueryDefinition = Q(ENEDIS_MINUTE_DOCTYPE)
      .where({
        month: {
          $eq: month,
        },
      })
      .indexFields(['year', 'month', 'day'])
      .sortBy([{ year: 'desc' }, { month: 'desc' }, { day: 'desc' }])
      .limitBy(1)
    const data = await client.query(query)
    return data.data
  }

  /**
   * Get the first entry of a given data doctype (enedis, grdf, egl)
   * @param doctype
   * @returns
   */
  public async getFirsDataDateFromDoctype(
    doctype: Doctype
  ): Promise<DataloadEntity[] | EnedisMonthlyAnalysisData[] | null> {
    const query: QueryDefinition = Q(doctype)
      .where({})
      .sortBy([{ year: 'asc' }, { month: 'asc' }])
      .limitBy(1)
    const data = await this._client.query(query)
    return data.data
  }

  public async getFirstDataDateFromDoctypeWithPrice(
    doctype: Doctype
  ): Promise<DataloadEntity | null> {
    const query: QueryDefinition = Q(doctype)
      .where({
        price: {
          $exists: false,
        },
      })
      .partialIndex({
        price: {
          $exists: false,
        },
      })
      .indexFields(['year', 'month'])
      .sortBy([{ year: 'asc' }, { month: 'asc' }])
      .limitBy(1)
    const data = await this._client.query(query)
    return data.data[0]
  }

  /**
   * Save one doc
   * @param {DataloadEntity} consumptionDoc - Doc to save
   * @returns {DataloadEntity} Saved doc
   */
  public async saveDoc(
    consumptionDoc: DataloadEntity
  ): Promise<DataloadEntity> {
    const { data: savedDoc }: QueryResult<DataloadEntity> =
      await this._client.save(consumptionDoc)
    return savedDoc
  }

  /**
   * Save an array of docs
   * @param {DataloadEntity[]} consumptionDocs - Array of doc to save
   * @returns {DataloadEntity[]} Array of saved docs
   */
  public async saveDocs(
    consumptionDocs: DataloadEntity[]
  ): Promise<DataloadEntity[]> {
    const { data: savedDocs }: QueryResult<DataloadEntity[]> =
      await this._client.saveAll(consumptionDocs)

    return savedDocs
  }
}
