import * as Sentry from '@sentry/react'
import { Client, Q, QueryDefinition, QueryResult } from 'cozy-client'
import logger from 'cozy-logger'
import {
  ENEDIS_MAXPOWER_DOCTYPE,
  ENEDIS_MONTHLY_ANALYSIS_DATA_DOCTYPE,
} from 'doctypes'
import { DataloadState } from 'enum/dataload.enum'
import { DateTime } from 'luxon'
import { Dataload } from 'models'
import {
  AggregatedEnedisMonthlyDataloads,
  EnedisMonthlyAnalysisData,
} from 'models/enedisMonthlyAnalysis'
import { MaxPowerEntity } from 'models/maxPower.model'
import logApp from 'utils/logger'

const logStack = logger.namespace('enedisMonthlyAnalysisDataService')
export default class EnedisMonthlyAnalysisDataService {
  private readonly _client: Client

  constructor(_client: Client) {
    this._client = _client
  }

  /**
   * Retrieve all exploration entities from db
   * @returns {EnedisMonthlyAnalysisData[]}
   */
  public async getAllEnedisMonthlyAnalysisData(): Promise<
    EnedisMonthlyAnalysisData[]
  > {
    const query: QueryDefinition = Q(ENEDIS_MONTHLY_ANALYSIS_DATA_DOCTYPE)
    const {
      data: enedisMonthlyAnalysisData,
    }: QueryResult<EnedisMonthlyAnalysisData[]> = await this._client.query(
      query
    )
    return enedisMonthlyAnalysisData
  }

  /**
   * getLastEnedisMonthlyAnalysis
   * @param {Client} client
   * @returns {Promise<EnedisMonthlyAnalysisData[]>}
   */
  public async getLastEnedisMonthlyAnalysis(): Promise<
    EnedisMonthlyAnalysisData[]
  > {
    const query: QueryDefinition = Q(ENEDIS_MONTHLY_ANALYSIS_DATA_DOCTYPE)
      .where({})
      .sortBy([{ year: 'desc' }, { month: 'desc' }])
      .limitBy(1)
    const data = await this._client.query(query)
    return data.data
  }

  /**
   * Aggregates Enedis Analysis data in order to create Dataload injectable in graph component
   * @param {EnedisMonthlyAnalysisData} data
   * @returns {AggregatedEnedisMonthlyDataloads}
   */
  public aggregateValuesToDataLoad = (
    data: EnedisMonthlyAnalysisData
  ): AggregatedEnedisMonthlyDataloads => {
    const dataLoadWeekDays: Dataload[] = []
    const dataLoadWeekEndDays: Dataload[] = []
    data.weekDaysHalfHourAverageValues.forEach((value, index) => {
      dataLoadWeekDays.push({
        value: value,
        state: DataloadState.VALID,
        valueDetail: null,
        date: DateTime.fromObject({
          year: data.year,
          month: data.month,
          minute: 0,
        })
          .setZone('utc', {
            keepLocalTime: true,
          })
          .plus({ minute: 30 * index }),
      })
    })
    data.weekEndDaysHalfHourAverageValues.forEach((value, index) => {
      dataLoadWeekEndDays.push({
        value: value,
        state: DataloadState.VALID,
        valueDetail: null,
        date: DateTime.fromObject({
          year: data.year,
          month: data.month,
          minute: 0,
        })
          .setZone('utc', {
            keepLocalTime: true,
          })
          .plus({ minute: 30 * index }),
      })
    })
    return {
      week: dataLoadWeekDays,
      weekend: dataLoadWeekEndDays,
    }
  }

  /**
   * Get an enedis monthly analysis for given month and year
   * @param {number} year
   * @param {number} month
   * @returns {Promise<EnedisMonthlyAnalysisData[]>}
   */
  public async getEnedisMonthlyAnalysisByDate(
    year: number,
    month: number
  ): Promise<EnedisMonthlyAnalysisData[]> {
    const query: QueryDefinition = Q(ENEDIS_MONTHLY_ANALYSIS_DATA_DOCTYPE)
      .where({ year: year, month: month })
      .sortBy([{ year: 'desc' }, { month: 'desc' }])
      .limitBy(1)
    const data = await this._client.query(query)

    return data.data
  }

  /**
   * Creates a new EnedisMonthlyAnalysis
   * @param {EnedisMonthlyAnalysisData} newEnedisMonthlyAnalysisData
   * @returns {Promise<EnedisMonthlyAnalysisData | null>}
   */
  public async createEnedisMonthlyAnalysisData(
    newEnedisMonthlyAnalysisData: EnedisMonthlyAnalysisData
  ): Promise<EnedisMonthlyAnalysisData | null> {
    try {
      const {
        data: EnedisMonthlyAnalysis,
      }: QueryResult<EnedisMonthlyAnalysisData> = await this._client.create(
        ENEDIS_MONTHLY_ANALYSIS_DATA_DOCTYPE,
        newEnedisMonthlyAnalysisData
      )
      return EnedisMonthlyAnalysis
    } catch (error) {
      const errorMessage = `Error creating new EnedisMonthlyAnalysis: ${JSON.stringify(
        error
      )}`
      logStack('error', errorMessage)
      logApp.error(errorMessage)
      Sentry.captureException(errorMessage)
      throw error
    }
  }

  /**
   * Get Max power for a given month and year
   * @param {number} year
   * @param {number} month
   * @returns {Promise<MaxPowerEntity[]>}
   */
  public async getMaxPowerByDate(
    year: number,
    month: number
  ): Promise<MaxPowerEntity[]> {
    const query: QueryDefinition = Q(ENEDIS_MAXPOWER_DOCTYPE)
      .where({ year: year, month: month })
      .sortBy([{ year: 'desc' }, { month: 'desc' }])
    const data = await this._client.query(query)

    return data.data
  }
}
